import socket   # TCP inter-process socket
import ctypes   # An included library with Python install.
import sys
import math


class PythonClass:
    """
    Example Python class with initializer, a static method that uses win_message to show text
    and an instance method that prints information about an instance of Python class
    """
    def __init__(self, init_variable):
        self.class_variable = init_variable

    @staticmethod
    def static_method():
        win_message("Hello from static_method()")

    def instance_method(self):
        print("PythonClass instance class_variable is set to:" + str(self.class_variable))


class ATEasyStdout:
    """
    Class with initializer, write and dummy flush method. Initializing and storing an instance of this class to
    sys.stdout will redirect to sys.stdout to instance.
    """
    def __init__(self):
        self.out = ""

    def write(self, write_data):
        self.out = self.out + write_data

    def flush(self):
        pass


def use_ateasy_stdout():
    """
    Method that uses ATEasyStdout to redirect stdout to an instance of ATEasyStdout.
    :return: Original sys.stdout stream (to restore the stdout stream)
    """
    restore_sys_stdout = sys.stdout
    sys.stdout = ATEasyStdout()
    return restore_sys_stdout


def connect_and_confirm_ateasy(host, port, input_data=0):
    """
    Calling function that opens a socket to host and either sends a ping message or
    processes data with receive_and_process_ateasy function
    :param host: 127.0.0.1
    :param port: 10
    :param input_data: State {0 -> send connection established message to opened socket ; 1 -> call a method to
    receive data via socket, compute factorial of received data and send the factorial back to sender}
    :return:
    """
    sck = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sck.connect((host, port))
    if input_data == 1:
        receive_and_process_ateasy(sck)
    else:
        sck.send(b'Python: Winsock connection established')
    return sck


def receive_and_process_ateasy(sck):
    """
    Called function. Called by connect_and_confirm_ateasy. Take factorial of received data and send it back with
    platform-dependent encoding
    :param sck: Open socket
    :return: Factorial of received data
    """
    rx_data = int(sck.recv(4))
    rx_data = math.factorial(rx_data)
    sck.send(str(rx_data).encode())
    return rx_data


class ATEasyStdin:
    """
    Class with initializer, write and read methods. Initializer sets some needed parameters for calling input() with it.
    This class is used to redirect and write standard input so that input() calls succeed with input from ATEasy.
    """

    def __init__(self, filenumber):
        self.input = ""
        self.fileno = filenumber
        self.encoding = 'utf-8'
        self.errors = None

    def fileno(self):
        return self.fileno

    def write(self, write_input):
        self.input = self.input + write_input

    def readline(self):
        return str(self.input)

    def read(self):
        return str(self.input)

    def flush(self):
        pass


def use_ateasy_stdin():
    """
    Method that uses ATEasyStdin to redirect stdin to an instance of ATEasyStdin.
    :return: sys.stdin stream
    """
    restore_sys_stdin = sys.stdin
    sys.stdin = ATEasyStdin(0)
    sys.stderr = ATEasyStdin(2)
    return sys.stdin


def echo_input():
    """
    Function that echoes input()
    :return: standard input
    """
    try:
        return input()
    except Exception as e:
        win_message(str(e))


def win_message(message):
    """
    Windows message box
    :param message: String message to show in the message box
    :return: If non-empty message was provided, return status message. If message string is empty return magic number 42
    """
    if message != "":
        ctypes.windll.user32.MessageBoxW(0, "Hello, ATEasy!\n\n"+message+"\n\nRegards,"
                                            "\nImported Python.py module ",
                                         "Greetings from Python.py module", 0)
        return "win_message message processed"
    else:
        ctypes.windll.user32.MessageBoxW(0, "Hello, ATEasy!\n\nRegards,"
                                            "\nImported Python.py module",
                                         "Greetings from Python.py module", 0)
        return 42
